use local_ip_address::local_ip;
use std::ops::Deref;
use std::sync::{Arc, Mutex};
use tide::prelude::*;
use tide::{Body, Request};

use lcd_writer::lcd::Lcd;

#[derive(Clone)]
struct App {
    values: Arc<Mutex<Vec<i32>>>,
    lcd: Arc<Mutex<Lcd>>,
}

impl App {
    fn new(lcd: Lcd) -> Self {
        App {
            values: Arc::new(Mutex::new(Vec::new())),
            lcd: Arc::new(Mutex::new(lcd)),
        }
    }
}

#[derive(Debug, Deserialize)]
struct Sample {
    value: i32,
}

#[async_std::main]
async fn main() -> tide::Result<()> {
    let lcd = Lcd::new()?;
    let mut app = tide::with_state(App::new(lcd));
    app.with(tide::log::LogMiddleware::new());
    app.at("/samples").get(get_samples);
    app.at("/sample").post(post_sample);
    app.at("/").serve_dir("public/")?;
    app.at("/").serve_file("public/index.html")?;
    if let Ok(ip) = local_ip() {
        println!("Starting server at {ip}:8000!");
    } else {
        println!("Starting server on port 8000. Could not get IP");
    }
    app.listen("0.0.0.0:8000").await?;
    Ok(())
}

async fn get_samples(req: Request<App>) -> tide::Result<Body> {
    let values = req.state().values.lock().unwrap();
    Body::from_json(values.deref())
}

async fn post_sample(mut req: Request<App>) -> tide::Result<Body> {
    let Sample { value } = req.body_json().await?;

    let mut lcd = req.state().lcd.lock().unwrap();
    lcd.clear()?;
    lcd.set_cursor(0, 0)?;
    lcd.write(format!("Value: {}", value).as_str())?;

    req.state().values.lock().unwrap().push(value);

    println!("Ok, value: {}", value);
    Body::from_json(&json!({"ok": true}))
}
